;;;; -*- Mode: LISP; Syntax: ANSI-Common-Lisp; Base: 10; Package: photo -*-
;;;; *************************************************************************
;;;; FILE IDENTIFICATION
;;;;
;;;; Name:          cameras.lisp
;;;; Purpose:       Camera-specific data for cl-photo
;;;; Programmer:    Kevin M. Rosenberg
;;;; Date Started:  April 2005
;;;;
;;;; $Id: dof.lisp 10421 2005-04-19 21:57:00Z kevin $
;;;;
;;;; This file, part of cl-photo, is Copyright (c) 2005 by Kevin M. Rosenberg
;;;;
;;;; cl-photo users are granted the rights to distribute and use this software
;;;; as governed by the terms of the GNU General Public License v2
;;;; (http://www.gnu.org/licenses/gpl.html)
;;;;
;;;; *************************************************************************

(in-package #:photo)

(defun sensor-dimensions-megapixels (format megapixels)
  (let* ((dim (imager-dimensions format))
         (aspect-ratio (/ (car dim) (cdr dim)))
         (width (round (sqrt (* aspect-ratio 1000000 megapixels)))))
    (cons width (round (/ width aspect-ratio)))))

(eval-when (:compile-toplevel :load-toplevel :execute)
  (defun make-output-format (w h &key (units :inches))
    (let ((name (format nil "~Dx~Din" w h)))
      (list :format (kmrcl:ensure-keyword name)
            :output (cons (inches->mm w) (inches->mm h))
            :name name
            :nicks (list (kmrcl:ensure-keyword (format nil "~Dx~D" w h)))))))

(defparameter +format-db+
  '(
   (:format :d2x :make "Nikon" :pixels (4288 . 2848) :imager (23.7 . 15.6) :name "D2X")
   (:format :d100 :make "Nikon" :pixels (3037 . 2024) :imager (23.7 . 15.6) :name "D100")
   (:format :d70 :make "Nikon" :pixels (3008 . 2000) :imager (23.7 . 15.6) :name "D70")
   (:format :d70s :make "Nikon" :pixels (3008 . 2000) :imager (23.7 . 15.6) :name "D70s")
   (:format :d50 :make "Nikon" :pixels (3008 . 2000) :imager (23.7 . 15.6) :name "D70")
   (:format :d1x :make "Nikon" :pixels (4028 . 1324) :imager (23.7 . 15.6) :name "D1X")
   (:format :d2h :make "Nikon" :pixels (2464 . 1632) :imager (23.3 . 15.5) :name "D2H")
   (:format :d2hs :make "Nikon" :pixels (2464 . 1632) :imager (23.3 . 15.5) :name "D2Hs")
   
   (:format :300d :make "Canon" :pixels (3072 . 2048) :imager (22.7 . 15.1) :name "300D")
   (:format :10d :make "Canon" :pixels (3072 . 2048) :imager (22.7 . 15.1) :name "10D")
   (:format :d30 :make "Canon" :pixels (0 . 0) :imager (22.7 . 15.1) :name "D30")
    (:format :d60 :make "Canon" :pixels (3072 . 2048) :imager (22.7 . 15.1) :name "D60")
   (:format :350d :make "Canon" :pixels (3456 . 2304) :imager (22.2 . 14.8) :name "350D")
   (:format :1d :make "Canon" :pixels (2464 . 1648) :imager (27 . 17.8) :name "1D")
   (:format :20d :make "Canon" :pixels (3456 . 2304) :imager (22.5 . 15) :name "20D")
   (:format :1dmkii :make "Canon" :pixels (3504 . 2336) :imager (28.7 . 19.1)
    :name "1D Mark II" :nicks (:1d2 :1dii :1dmkii))
   (:format :1ds :make "Canon" :pixels (4064 . 3328) :imager (36 . 24) :name "1Ds")
   (:format :1dsmkii :make "Canon" :pixels (4992 . 3328) :imager (36 . 24)
    :name "1Ds Mark II" :nicks (:1ds2 :1dsii))

   (:format :35mm :imager (36 . 24) :name "35mm")

   (:format :6x4.5cm :imager (60 . 45) :name "6x4.5cm" :nicks (:\645))
   (:format :6x6cm :imager (60 . 60) :name "6x6cm" :nicks (:6x6))
   (:format :6x7cm :imager (60 . 70) :name "6x7cm" :nicks (:6x7))
   (:format :6x9cm :imager (60 . 90) :name "6x9cm" :nicks (:6x9))
   (:format :6x12cm :imager (60 . 120) :name "6x12cm" :nicks (:6x12))

   #.(make-output-format 4 5)
   #.(make-output-format 5 7)
   #.(make-output-format 8 10)
   #.(make-output-format 11 13.75)
   #.(make-output-format 11 16.5)
   #.(make-output-format 13 16.25)
   #.(make-output-format 13 19)
   #.(make-output-format 16 20)
   #.(make-output-format 16 24)
   #.(make-output-format 18 22.5)
   #.(make-output-format 18 24)
   #.(make-output-format 24 30)
   #.(make-output-format 24 36)
  ))

(defun format-match-p (format-spec format)
  (when (or (eql format-spec (getf format :format))
            (member format-spec (getf format :nicks)))
    t))

(defun find-format (format-spec)
  (find format-spec +format-db+ :test 'format-match-p))

(defun pixel-dimensions (sensor-spec &key (format :35mm))
  "Returns the number of pixels for a format. 
CAMERA-SPEC is either a keyword designating the camera or
the number of megapixels of the sensor.
FORMAT should be defined if the CAMERA-SPEC is the number of megapixels
so the proper aspect ratio is used."
  (etypecase sensor-spec
    (keyword
     (getf (find-format sensor-spec) :pixels))
    (number
     (sensor-dimensions-megapixels format sensor-spec))))

(defun imager-dimensions (format-spec)
  "Returns the imager dimensions in mm of a FORMAT."
  (getf (find-format format-spec) :imager))

(defun output-dimensions (format-spec)
  "Returns the output dimensions in mm of a FORMAT."
  (getf (find-format format-spec) :output))
